use std::collections::HashMap;

use big_s::S;
use shapemaker::{AudioSyncPaths, Canvas, CenterAnchor, Color, ColorMapping, Fill, Object, Video};

struct Objects {
    bass: Object,
}

impl Default for Objects {
    fn default() -> Self {
        Self {
            bass: Object::BigCircle(CenterAnchor(0, 0)),
        }
    }
}

fn main() {
    let mut canvas = Canvas::default_settings();
    canvas.grid_size = (16, 9);
    canvas.colormap = ColorMapping {
        black: S("#000"),
        white: S("#fff"),
        blue: S("#003c8f"),
        green: S("#43A047"),
        purple: S("#9c27b0"),
        brown: S("#795548"),
        orange: S("#ef6c00"),
        gray: S("#a7c0cd"),
        cyan: S("#039BE5"),
        yellow: S("#ffb300"),
        pink: S("#F48FB1"),
        red: S("#E53935"),
        // some additional colors that could be used
        // #26A69A
        // #62EFFF
    };

    let midi_mappings = HashMap::from([
        ("powerful clap", "powerful clap hit"),
        ("brokenup", "erodedmain"),
    ]);

    Video::<Objects>::new()
        .sync_to(&AudioSyncPaths {
            stems: S("stems/"),
            landmarks: S("landmarks.json"),
            bpm: S("bpm.txt"),
            complete: S("full.mp3"),
            midi: S("full.midi"),
        }, midi_mappings)
        .set_fps(60)
        .set_initial_canvas(canvas)
        .init(&|canvas, _| {
            canvas.background = Some(Color::White);
        })
        .each_frame(&|canvas, ctx| {
            canvas.add_object(
                "root",
                "debuginfo",
                Object::RawSVG(Box::new(
                    svg::node::element::Text::new()
                        .set("x", 10)
                        .set("y", 10)
                        .set("font-size", 10)
                        .set("fill", "black")
                        .set("font-family", "monospace")
                        .add(svg::node::Text::new(format!(
                            "amplitudes: bass: {:.3} goup: {:.3} qanda: {:.3}",
                            ctx.stem("bass").amplitude_relative(),
                            ctx.stem("goup").amplitude_relative(),
                            ctx.stem("qanda").amplitude_relative(),
                        ))),
                )),
                None,
            );
        })
        .on_stem(
            "anchor kick",
            0.8,
            &|canvas, _| {
                canvas.background = Some(Color::Gray);
            },
            &|canvas, _| {
                canvas.background = Some(Color::White);
            },
        )
        .on_stem(
            "bass",
            0.75,
            &|canvas, ctx| {
                canvas.add_object(
                    "root",
                    "bass",
                        Object::BigCircle(canvas.random_center_anchor()),
                        Some(Fill::Solid(Color::Brown)),
                );
            },
            &|canvas, ctx| {},
        )
        .on_stem(
            "brokenup",
            0.3,
            &|canvas, ctx| {
                canvas.add_object(
                    "root",
                    "brokenup",
                    canvas.random_polygon(), Some(Fill::Solid(Color::Orange)),
                );
            },
            &|canvas, ctx| {
                canvas.remove_object("brokenup");
            },
        )
        .on_stem(
            "goup",
            0.3,
            &|canvas, ctx| {
                canvas.add_object(
                    "root",
                    "goup",
                    canvas.random_polygon(), Some(Fill::Solid(Color::Yellow)),
                );
            },
            &|canvas, ctx| {
                canvas.remove_object("goup");
            },
        )
        .on_stem(
            "qanda",
            0.3,
            &|canvas, ctx| {
                canvas.add_object(
                    "root",
                    "qanda",
                    canvas.random_polygon(), Some(Fill::Solid(Color::Red)),
                );
            },
            &|canvas, ctx| {
                canvas.remove_object("qanda");
            },
        )
        .on_stem(
            "ch",
            0.5,
            &|canvas, ctx| {
                canvas.add_object(
                    "root",
                    "ch",
                        Object::Dot(canvas.random_anchor()),
                        Some(Fill::Solid(Color::Gray)),
                );
            },
            &|canvas, ctx| {
                canvas.remove_object("ch");
            },
        )
        .on_stem(
            "clap",
            0.3,
            &|canvas, ctx| {
                canvas.add_object(
                    "root",
                    "clap",
                        Object::Line(canvas.random_anchor(), canvas.random_anchor()),
                        Some(Fill::Solid(Color::Green)),
                );
            },
            &|canvas, ctx| {
                canvas.remove_object("clap");
            },
        )
        .on_stem(
            "perclap",
            0.3,
            &|canvas, ctx| {
                canvas.add_object(
                    "root",
                    "perclap",
                        Object::Line(canvas.random_anchor(), canvas.random_anchor()),
                        Some(Fill::Solid(Color::Cyan)),
                );
            },
            &|canvas, ctx| {
                canvas.remove_object("perclap");
            },
        )
        .on_stem(
            "glitchy percs",
            0.3,
            &|canvas, ctx| {
                canvas.add_object(
                    "root",
                    "glitchy percs",
                    Object::SmallCircle(canvas.random_anchor()),
                    Some(Fill::Solid(Color::Pink)),
                );
            },
            &|canvas, ctx| {
                canvas.remove_object("glitchy percs");
            },
        )
        .on_stem(
            "hitting percs",
            0.3,
            &|canvas, ctx| {
                canvas.add_object(
                    "root",
                    "hitting percs",
                    Object::SmallCircle(canvas.random_anchor()),
                    Some(Fill::Solid(Color::Purple)),
                );
            },
            &|canvas, ctx| {
                canvas.remove_object("hitting percs");
            },
        )
        .command("bpm", &|arguments, canvas, context| {
            let bpm = arguments.parse::<usize>().unwrap();
            context.bpm = bpm;
        })
        .render_to(S("../schedule-hell.mp4"), 8);
}
